# -*- coding: utf-8 -*-
# author:navysummer
# email:yuyc@ishangqi.com
# date:2021/11/26 3:56 下午
# Copyright (C) 2021 The lesscode Team
from functools import reduce

from nebula3.data.DataObject import Value, ValueWrapper, Relationship, PathWrapper, Node, Null
from nebula3.data.ResultSet import ResultSet
from nebula3.mclient import MetaCache, HostAddr
from nebula3.sclient.GraphStorageClient import GraphStorageClient
from tornado.options import options


class NebulaHelper:
    def __init__(self, pool):
        """
        初始化sql工具
        :param pool: 连接池名称
        """
        if isinstance(pool, str):
            self.pool, self.conn_info = options.database[pool]
        else:
            self.pool, self.conn_info = pool, None

    def exec_gql(self, sql, space=None):
        space = space if space else self.conn_info.db_name
        if not space:
            raise Exception(f"nebula no selection space")
        with self.pool.session_context(self.conn_info.user, self.conn_info.password) as session:
            session.execute(f'USE {space}')
            result = session.execute(sql)
        return result

    def fetch_data(self, sql, space=None):
        result = self.exec_gql(sql, space)
        result = convert(result)
        return result


class NebulaStorageHelper:
    def __init__(self, meta_cache_config: dict = None, storage_address_config: dict = None,
                 graph_storage_client_timeout=60000):
        self.meta_cache_config = meta_cache_config
        self.storage_address_config = storage_address_config
        self.graph_storage_client_timeout = graph_storage_client_timeout

    def get_meta_cache(self):
        meta_cache = None
        if self.meta_cache_config:
            meta_addrs = self.meta_cache_config.get("meta_addrs")
            timeout = self.meta_cache_config.get("timeout", 2000)
            load_period = self.meta_cache_config.get("load_period", 10)
            decode_type = self.meta_cache_config.get("decode_type", 'utf-8')
            meta_cache = MetaCache(meta_addrs, timeout, load_period, decode_type)
        return meta_cache

    def get_storage_addrs(self):
        storage_addrs = None
        storage_addrs = [HostAddr(host=sa.get("host"), port=sa.get("port")) for sa in storage_addrs]
        return storage_addrs

    def get_graph_storage_client(self):
        meta_cache = self.get_meta_cache()
        storage_addrs = self.get_storage_addrs()
        graph_storage_client = GraphStorageClient(meta_cache, storage_addrs, self.graph_storage_client_timeout)
        return graph_storage_client

    def scan_vertex(self, *args, **kwargs):
        graph_storage_client = self.get_graph_storage_client()
        resp = graph_storage_client.scan_vertex(*args, **kwargs)
        data = []
        while resp.has_next():
            result = resp.next()
            for vertex_data in result:
                data.append(vertex_data)
        return data

    def scan_edge(self, *args, **kwargs):
        graph_storage_client = self.get_graph_storage_client()
        resp = graph_storage_client.scan_edge(*args, **kwargs)
        data = []
        while resp.has_next():
            result = resp.next()
            for edge_data in result:
                data.append(edge_data)
        return data


cast_as = {
    Value.NVAL: "as_null",
    Value.__EMPTY__: "as_empty",
    Value.BVAL: "as_bool",
    Value.IVAL: "as_int",
    Value.FVAL: "as_double",
    Value.SVAL: "as_string",
    Value.LVAL: "as_list",
    Value.UVAL: "as_set",
    Value.MVAL: "as_map",
    Value.TVAL: "as_time",
    Value.DVAL: "as_date",
    Value.DTVAL: "as_datetime",
    Value.VVAL: "as_node",
    Value.EVAL: "as_relationship",
    Value.PVAL: "as_path",
    Value.GGVAL: "as_geography",
    Value.DUVAL: "as_duration"
}


def list_dict_duplicate_removal(data_list):
    run_function = lambda x, y: x if y in x else x + [y]
    return reduce(run_function, [[], ] + data_list)


def merge_nebula_value(data_list: list) -> dict:
    nodes = []
    relationships = []

    for data in data_list:
        nodes.extend(list(data.values())[0]["nodes"])
        relationships.extend(list(data.values())[0]["relationships"])
    result_dict = {"nodes": list_dict_duplicate_removal(nodes),
                   "relationships": list_dict_duplicate_removal(relationships)}
    return result_dict


def customized_cast_with_dict(val: ValueWrapper):
    _type = val._value.getType()
    method = cast_as.get(_type)
    if method is not None:
        value = getattr(val, method, lambda *args, **kwargs: None)()
        if isinstance(value, dict):
            for k, v in value.items():
                value[k] = customized_cast_with_dict(v)
        elif isinstance(value, list):
            for i, v in enumerate(value):
                value[i] = customized_cast_with_dict(v)
        elif isinstance(value, set):
            new_value = set()
            for v in value:
                new_value.add(customized_cast_with_dict(v))
            value = new_value
        elif isinstance(value, tuple):
            new_value = []
            for v in value:
                new_value.append(customized_cast_with_dict(v))
            value = tuple(new_value)
        elif isinstance(value, Relationship):
            value = {k: customized_cast_with_dict(v) for k, v in value.properties().items()}
        elif isinstance(value, PathWrapper):
            nodes = []
            relationships = []
            for node in value.nodes():
                vid = customized_cast_with_dict(node.get_id())
                for tag in node.tags():
                    point = {"_vid": vid, "_tag": tag}
                    point.update({k: customized_cast_with_dict(v) for k, v in node.properties(tag).items()})
                    nodes.append(point)
            for rel in value.relationships():
                relationship = {"_name": rel.edge_name(), "_start": customized_cast_with_dict(rel.start_vertex_id()),
                                "_end": customized_cast_with_dict(rel.end_vertex_id())}
                relationship.update({k: customized_cast_with_dict(v) for k, v in rel.properties().items()})
                relationships.append(relationship)
            value = {"nodes": nodes, "relationships": relationships}
        elif isinstance(value, Node):
            nodes = []
            for tag in value.tags():
                point = {"_vid": customized_cast_with_dict(value.get_id()), "_tag": tag}
                point.update({k: customized_cast_with_dict(v) for k, v in value.properties(tag).items()})
                nodes.append(point)
            value = {"nodes": nodes}
        elif isinstance(value, Null):
            value = None
        return value
    raise KeyError("No such key: {}".format(_type))


def convert(resp: ResultSet):
    assert resp.is_succeeded()
    value_list = []
    for recode in resp:
        record = recode
        if hasattr(recode, "keys"):
            record = {}
            for key in recode.keys():
                val = customized_cast_with_dict(recode.get_value_by_key(key))
                record[key] = val
        if record:
            value_list.append(record)
    return value_list



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/12/8 10:53 上午
# Copyright (C) 2021 The lesscode Team
import json
import logging
import os
import random
import traceback
from datetime import datetime

import requests
from requests.auth import HTTPBasicAuth
from tornado.options import options

from lesscode.db.base_sql_helper import BaseSqlHelper, echo_sql
from lesscode.db.condition_wrapper import ConditionWrapper
from lesscode.db.connection_info import ConnectionInfo
from lesscode.db.es.es_pool import EsPool
from lesscode.db.es.es_request import EsRequest
from lesscode.db.page import Page
from lesscode.utils.EsUtil import format_es_param_result
from lesscode.utils.encryption_algorithm import AES


class EsHelper:
    """
    ElasticsearchHelper  ES数据库操作实现
    """

    def __init__(self, pool):
        """
        初始化sql工具
        :param pool: 连接池名称
        """
        if isinstance(pool, str):
            self.pool, self.conn_info = options.database[pool]
        else:
            self.pool = pool

    async def send_es_post(self, bool_must_list=None, param_list=None, route_key="", sort_list=None, size=10,
                           offset=0,
                           track_total_hits=False):
        params = {
            "query": {
                "bool": {
                    "must": bool_must_list
                }
            },
            "size": size,
            "from": offset,
        }
        if track_total_hits:
            params["track_total_hits"] = track_total_hits
        if param_list:
            params["_source"] = {"include": param_list}
        if sort_list:
            params["sort"] = sort_list
        start_time = datetime.now()

        res = self.pool.es_selector_way(url_func_str=self.pool.format_es_post_url, param_dict={
            "route_key": route_key,
        }, find_condition=params)
        logging.info("进程{}，路由{},查询时间{}".format(os.getpid(), route_key, datetime.now() - start_time))

        if "error" in list(res.keys()):
            logging.info(res)
        return res["hits"]

    async def format_es_return(self, bool_must_list=None, param_list=None, route_key="", sort_list=None, size=10,
                               offset=0,
                               track_total_hits=False, is_need_es_score=False, is_need_decrypt_oralce=False, res=None):
        if not res:
            res = await self.send_es_post(bool_must_list, param_list, route_key=route_key, sort_list=sort_list,
                                          size=size,
                                          offset=offset,
                                          track_total_hits=track_total_hits)

        result_list = []
        for r in res["hits"]:
            result_list.append(
                format_es_param_result(r, param_list, is_need_decrypt_oralce, is_need_es_score, route_key))
        result_dict = {
            "data_count": res["total"]["value"],
            "data_list": result_list
        }
        return result_dict

    async def format_es_scan(self, bool_must_list=None, param_list=None, route_key="", scroll="5m", size=10000,
                             is_need_decrypt_oralce=False, limit=None):
        logging.info("扫描开始，条件是{},查询字段是{}".format(json.dumps(bool_must_list), json.dumps(param_list)))
        skip = 0
        request_param = {
            "query": {
                "bool": {
                    "must": bool_must_list
                }
            }
            , "size": size,
        }
        if param_list:
            request_param["_source"] = {"include": param_list}
        res = self.pool.es_selector_way(url_func_str=self.pool.format_scroll_url, param_dict={
            "route_key": route_key,
            "scroll": scroll
        }, find_condition=request_param)
        data_size = len(res["hits"]["hits"])
        logging.info(
            "扫描{}:{}条花费时间{}ms,".format(route_key, str(skip) + "-" + str(skip + data_size), res["took"]))
        scroll_id = res["_scroll_id"]
        result_list = []
        for data in res["hits"]["hits"]:
            if is_need_decrypt_oralce:
                data["_id"] = AES.encrypt(options.aes_key, data["_id"])
            data["_source"]["_id"] = data["_id"]
            result_list.append(data["_source"])
        while True:
            skip = skip + data_size

            res = self.pool.es_selector_way(url_func_str=self.pool.format_scroll_id_url, param_dict={
            }, find_condition={
                "scroll": scroll,
                "scroll_id": scroll_id})
            data_size = len(res["hits"]["hits"])
            logging.info(
                "扫描{}:{}条花费时间{}ms,".format(route_key, str(skip) + "-" + str(skip + data_size), res["took"]))
            scroll_id = res.get("_scroll_id")
            # end of scroll
            if scroll_id is None or not res["hits"]["hits"]:
                break
            for data in res["hits"]["hits"]:
                data["_source"]["_id"] = data["_id"]
                result_list.append(data["_source"])
            if limit and limit <= len(result_list):
                break
        return result_list

    async def format_es_group(self, bool_must_list=None, route_key="", aggs=None):
        params = {
            "query": {
                "bool": {
                    "must": bool_must_list
                }
            },
            "size": 0,
            "aggs": aggs
        }
        res = self.pool.es_selector_way(url_func_str=self.pool.format_es_post_url, param_dict={
            "route_key": route_key,
        }, find_condition=params)
        return res

    async def es_search(self, route_key, body):
        path = f"/{route_key}/_search"
        url = self.pool.format_url(path)
        res = self.pool.format_es_post(url, body=body)
        return res

    def sync_send_es_post(self, bool_must_list=None, param_list=None, route_key="", sort_list=None, size=10,
                          offset=0,
                          track_total_hits=False):
        params = {
            "query": {
                "bool": {
                    "must": bool_must_list
                }
            },
            "size": size,
            "from": offset,
        }
        if sort_list:
            params["sort"] = sort_list
        try:
            if options.correction_param.get("enable"):
                mappings = self.pool.es_selector_way(url_func_str=self.pool.format_es_mapping_url,
                                                     param_dict={"route_key": route_key},
                                                     request_way="get").get(route_key, {}).get("mappings", {}).get(
                    "properties", {})
                self.correction_param(params, mappings)
        except:
            logging.error(traceback.format_exc())
        if track_total_hits:
            params["track_total_hits"] = track_total_hits
        if param_list:
            params["_source"] = {"include": param_list}
        start_time = datetime.now()

        res = self.pool.es_selector_way(url_func_str=self.pool.format_es_post_url, param_dict={
            "route_key": route_key,
        }, find_condition=params)
        logging.info("进程{}，路由{},查询时间{}".format(os.getpid(), route_key, datetime.now() - start_time))

        if "error" in list(res.keys()):
            logging.info(res)
        return res["hits"]

    def correction_param(self, param, mappings):
        for k in param:
            if k in ["terms", "term", "range", "wildcard", "sort"]:
                param[k] = self.correction_field(param[k], mappings, keyword_flag=True)
            elif k in ["match", "match_phrase"]:
                param[k] = self.correction_field(param[k], mappings)
            else:
                if isinstance(param[k], dict):
                    self.correction_param(param[k], mappings)
                elif isinstance(param[k], list):
                    for i in param[k]:
                        self.correction_param(i, mappings)

    def correction_field(self, value, mappings, keyword_flag=False):
        if isinstance(value, dict):
            if value.get("field"):
                column = value["field"]
                column = self.check_replace_keyword(column, mappings, keyword_flag)
                value["field"] = column
                return value
            else:
                column = list(value.keys())[0]
                data_value = value[column]
                column = self.check_replace_keyword(column, mappings, keyword_flag)
                return {
                    column: data_value
                }
        elif isinstance(value, list):
            for index, i in enumerate(value):
                column = list(i.keys())[0]
                data_value = i[column]
                column = self.check_replace_keyword(column, mappings, keyword_flag)
                value[index] = {
                    column: data_value
                }
        return value

    def check_replace_keyword(self, column, mappings, keyword_flag):
        column = column.replace(".keyword", "")
        es_mapping_value = self.get_es_filed_mapping(column, mappings)
        if es_mapping_value:
            if es_mapping_value.get("text") and es_mapping_value.get("keyword"):
                if keyword_flag:
                    column = f'{column}.keyword'
        return column

    def get_es_filed_mapping(self, field, mappings):
        field_list = field.split(".")
        res = self.get_field_children_mapping(field_list, mappings)
        if res:
            if res.get("fields"):
                result = {
                    res["type"]: 1,
                    list(res["fields"].values())[0]["type"]: 1
                }
            else:
                result = {
                    res["type"]: 1
                }
        else:
            result = {}
        return result

    def get_field_children_mapping(self, field_list, mapping):
        if field_list:
            mapping = mapping.get(field_list[0])
            field_list = field_list[1:]
            if field_list:
                mapping = mapping.get("properties")
                return self.get_field_children_mapping(field_list, mapping)
            else:
                return mapping

    def get_mapping_text_and_keyword_column(self, mapping, reuslt, column=""):
        for key in mapping:
            # 获取value的key都有什么是否拥有type和fields
            value_mapping = mapping[key]
            if value_mapping.get("type") in ["text", "keyword"] and value_mapping.get("index", True):
                if value_mapping.get("fields"):
                    reuslt[f'{column}.{key}' if column else key] = {
                        "text": 1,
                        "keyword": 1
                    }
                else:
                    reuslt[f'{column}.{key}' if column else key] = {
                        value_mapping["type"]: 1
                    }
            elif isinstance(value_mapping.get("type"), str):
                continue
            else:
                self.get_mapping_text_and_keyword_column(mapping[key], reuslt,
                                                         (f'{column}.{key}' if column else key) if key not in [
                                                             "properties", "fields",
                                                             "keyword"] else column)

    def sync_format_es_return(self, bool_must_list=None, param_list=None, route_key="", sort_list=None, size=10,
                              offset=0,
                              track_total_hits=False, is_need_es_score=False, is_need_decrypt_oralce=False, res=None):
        if not res:
            res = self.sync_send_es_post(bool_must_list, param_list, route_key=route_key, sort_list=sort_list,
                                         size=size,
                                         offset=offset,
                                         track_total_hits=track_total_hits)

        result_list = []
        for r in res["hits"]:
            result_list.append(
                format_es_param_result(r, param_list, is_need_decrypt_oralce, is_need_es_score, route_key))
        result_dict = {
            "data_count": res["total"]["value"],
            "data_list": result_list
        }
        return result_dict

    def sync_format_es_scan(self, bool_must_list=None, param_list=None, route_key="", scroll="5m", size=10000,
                            is_need_decrypt_oralce=False, limit=0, offset=0, sort_list=None):
        logging.info("扫描开始，条件是{},查询字段是{}".format(json.dumps(bool_must_list), json.dumps(param_list)))
        skip = 0
        limit = offset + limit
        params = {
            "query": {
                "bool": {
                    "must": bool_must_list
                }
            }
            , "size": size
        }
        if sort_list:
            params["sort"] = sort_list
        try:
            if options.correction_param.get("enable"):
                mappings = self.pool.es_selector_way(url_func_str=self.pool.format_es_mapping_url,
                                                     param_dict={"route_key": route_key},
                                                     request_way="get").get(route_key, {}).get("mappings", {}).get(
                    "properties", {})
                self.correction_param(params, mappings)
        except:
            logging.error(traceback.format_exc())
        if param_list:
            params["_source"] = {"include": param_list}
        res = self.pool.es_selector_way(url_func_str=self.pool.format_scroll_url, param_dict={
            "route_key": route_key,
            "scroll": scroll
        }, find_condition=params)
        data_size = len(res["hits"]["hits"])
        logging.info(
            "扫描{}:{}条花费时间{}ms,".format(route_key, str(skip) + "-" + str(skip + data_size), res["took"]))
        scroll_id = res["_scroll_id"]
        result_list = []
        for data in res["hits"]["hits"]:
            if is_need_decrypt_oralce:
                data["_id"] = AES.encrypt(options.aes_key, data["_id"])
            data["_source"]["_id"] = data["_id"]
            result_list.append(data["_source"])
        while True:
            skip = skip + data_size

            res = self.pool.es_selector_way(url_func_str=self.pool.format_scroll_id_url, param_dict={
            }, find_condition={
                "scroll": scroll,
                "scroll_id": scroll_id})
            data_size = len(res["hits"]["hits"])
            logging.info(
                "扫描{}:{}条花费时间{}ms,".format(route_key, str(skip) + "-" + str(skip + data_size), res["took"]))
            scroll_id = res.get("_scroll_id")
            # end of scroll
            if scroll_id is None or not res["hits"]["hits"]:
                break
            for data in res["hits"]["hits"]:
                if is_need_decrypt_oralce:
                    data["_id"] = AES.encrypt(options.aes_key, data["_id"])
                data["_source"]["_id"] = data["_id"]
                result_list.append(data["_source"])
            if limit and limit <= len(result_list):
                break
        return result_list[offset:offset + limit]

    def sync_format_es_group(self, bool_must_list=None, route_key="", aggs=None):
        params = {
            "query": {
                "bool": {
                    "must": bool_must_list
                }
            },
            "size": 0,
            "aggs": aggs
        }
        try:
            if options.correction_param.get("enable"):
                mappings = self.pool.es_selector_way(url_func_str=self.pool.format_es_mapping_url,
                                                     param_dict={"route_key": route_key},
                                                     request_way="get").get(route_key, {}).get("mappings", {}).get(
                    "properties", {})
                self.correction_param(params, mappings)
        except:
            logging.error(traceback.format_exc())

        res = self.pool.es_selector_way(url_func_str=self.pool.format_es_post_url, param_dict={
            "route_key": route_key,
        }, find_condition=params)
        if "error" in list(res.keys()):
            logging.info(res)
        return res

    def sync_delete_one(self, route_key, id):
        path = f"/{route_key}/{id}"
        url = self.pool.format_url(path)
        res = self.pool.format_es_delete(url=url)
        return res

    def sync_delete_data(self, route_key, params=[]):
        path = f"/{route_key}/_delete_by_query"
        url = self.pool.format_url(path)
        data = {
            "query": {
                "bool": {
                    "must": params
                }
            }
        }
        res = self.pool.format_es_delete(url=url, data=data)
        return res

    def sync_update_data(self, route_key, params, new_data: dict):
        path = f"/{route_key}/_update_by_query"
        url = self.pool.format_url(path)
        data = {
            "query": {
                "bool": {
                    "must": params
                }
            },
            "script": {
                "source": ""
            }
        }
        source = ""
        for k, v in new_data.items():
            source += f'ctx._source.{k}={v};'
        data["script"]["source"] = source
        res = self.pool.format_es_post(url=url, body=data)
        return res

    def sync_update_one(self, route_key, id, data):
        path = f"/{route_key}/{id}/_update"
        url = self.pool.format_url(path)
        res = self.pool.format_es_post(url, body=data)
        return res

    def sync_insert_one(self, route_key, data, id=None):
        if id:
            path = f"/{route_key}/{id}"
        else:
            path = f"/{route_key}"
        url = self.pool.format_url(path)
        res = self.pool.format_es_post(url, body=data)
        return res

    def sync_insert_data(self, route_key, data):
        path = f"/{route_key}/_bulk"
        url = self.pool.format_url(path)
        res = self.pool.format_es_post(url, body=data)
        return res

    def sync_es_search(self, route_key, body):
        path = f"/{route_key}/_search"
        url = self.pool.format_url(path)
        res = self.pool.format_es_post(url, body=body)
        if "error" in res:
            raise Exception(f"res={res}")
        return res

    def sync_es_search_format_hits(self, route_key, body):
        res = self.sync_es_search(route_key, body)
        hits = res.get("hits", {})
        data_count = hits.get("total", {}).get("value", 0)
        data_list = hits.get("hits", [])
        result = {
            "data_count": data_count,
            "data_list": data_list
        }
        return result

    def sync_es_search_format_aggregations(self, route_key, body):
        res = self.sync_es_search(route_key, body)
        hits = res.get("hits", {})
        data_count = hits.get("total", {}).get("value", 0)
        data_list = hits.get("hits", [])
        aggregations = res.get("aggregations")
        result = {
            "data_count": data_count,
            "data_list": data_list,
            "aggregations": aggregations
        }
        return result

    def sync_es_scan(self, route_key, body, scroll="5m"):
        path = f"/{route_key}/_search?scroll={scroll}"
        url = self.pool.format_url(path)
        res = self.pool.format_es_post(url, body=body)
        data_list = []
        if "error" in res:
            raise Exception(f"res={res}")
        hits = res.get("hits", {}).get("hits", [])
        for x in hits:
            data_list.append(x)
        scroll_id = res.get("_scroll_id")
        scroll_path = f"/_search/scroll"
        scroll_url = self.pool.format_url(scroll_path)
        while scroll_id:
            res = self.pool.format_es_post(scroll_url, body={'scroll': scroll, 'scroll_id': scroll_id})
            if "error" in res:
                raise Exception(f"res={res}")
            hits = res.get("hits", {}).get("hits", [])
            scroll_id = res.get("_scroll_id")
            if hits:
                for x in hits:
                    data_list.append(x)
            else:
                break
        data_count = len(data_list)
        result = {
            "data_count": data_count,
            "data_list": data_list
        }
        return result

    def sync_es_mapping(self, route_key, field: str = None):
        path = f"/{route_key}/_mapping"
        if field:
            path = f"{path}/field/{field}"
        url = self.pool.format_url(path)
        res = self.pool.format_es_get(url)
        return res

    def sync_es_request(self, path, method, params=None, body=None):
        url = self.pool.format_url(path)
        method = method.lower()
        if method in ["get", "post", "put", "delete"]:
            res = None
            if method == "get":
                res = self.pool.format_es_get(url, params)
            elif method == "post":
                res = self.pool.format_es_post(url, body, params)
            elif method == "put":
                res = self.pool.format_es_put(url, body, params)
            elif method == "delete":
                res = self.pool.format_es_delete(url, body, params)
            return res
        else:
            raise Exception(f"method={method} is not supported")



from lesscode.utils.common import get_value_from_dict


def es_list2table(title: str, head, data, column_keys: dict = None,
                  data_key: str = "data_list", count_key="data_count", index_dict: dict = None):
    """
    es的查询结果转换成表格
    :param index_dict: 索引列信息，实例：{"start":1,"column_name":"序号"，"index"："index"，"key"："index"}
    :param count_key: 统计字段的key
    :param data_key: 数据字段的key
    :param column_keys:数据对应点的keys，实例1：{"name1":"name2"} 说明：name1：接口返回字段名，name2:是数据的key,支持多层key，通过.连接，例如：basic.name
    :param data: es返回的数据
    :param title: 表格标题，字符串类型
    :param head: 表头，支持dict和list ，实例1：{"企业名称": "name"},实例2：[{"企业名称": "name"}]

    """

    result = {
        "title": title,
        "columns": [],
        "dataSource": [],
        "total": get_value_from_dict(data, count_key)
    }
    if isinstance(head, dict):
        for key, value in head.items():
            result["columns"].append({"title": key,
                                      "dataIndex": value, "key": value})
    elif isinstance(head, list):
        result["columns"] = head
    else:
        raise Exception(f'head={head} is error')
    if data_key:
        data_list = get_value_from_dict(data, data_key)
    else:
        data_list = data
    if column_keys:
        for item in data_list:
            data_item = {
            }
            for key, value in column_keys.items():
                data_item[key] = get_value_from_dict(item, value)
            result["dataSource"].append(data_item)
    else:
        result["dataSource"] = data_list

    if index_dict:
        ix = index_dict.get("start", 1)
        index_title = index_dict.get("column_name", "序号")
        data_index = index_dict.get("index", "index")
        data_key = index_dict.get("key", "index")
        result["columns"].append({"title": index_title,
                                  "dataIndex": data_index, "key": data_key})
        for item in result["dataSource"]:
            item[data_key] = ix
            ix += 1

    return result


def es_group2tree(data, group_list, key_name="key", index=0):
    result = []
    group = group_list[index]
    if group in data.keys():
        buckets = data.get(group, {}).get("buckets", [])
        for bucket in buckets:
            info = {
                "key": bucket.get(key_name),
                "doc_count": bucket.get("doc_count"),
                "group": group,
                "children": []
            }
            if index + 1 < len(group_list):
                g = group_list[index + 1]
                if g in bucket:
                    info["children"] = es_group2tree(bucket, group_list, key_name, index + 1)
            result.append(info)
    return result


def covert2percentage(data, key="doc_count", flag=True, children_key="children", total=0):
    if isinstance(data, list):
        for x in data:
            doc_count = x.get(key, 0)
            total += doc_count
            if children_key in x:
                x[children_key] = covert2percentage(x[children_key], key, flag, children_key, doc_count)
        for y in data:
            percentage = round(y.get("doc_count", 0) / total, 2) if total else 0
            y["percentage"] = percentage * 100 if flag else percentage
    return data


def convert_key_by_map(data, mapping: dict):
    if isinstance(data, list):
        for x in data:
            convert_key_by_map(x, mapping)
    elif isinstance(data, dict):
        for k, v in mapping.items():
            if k in data:
                data[v] = data[k]
                data.pop(k)


def es_single_group2ring(title: str, data, group_type: str, data_key="aggregations",
                         unit="%", second_unit="", flag=True):
    """
    es单分组转环形图
    :param title: 图题
    :param data: es分组原始数据
    :param group_type: 分组类型
    :param data_key: 数据key
    :param unit: 百分比单位
    :param second_unit: 数值单位
    :param flag: 百分比是否*100
    :return:
    """
    group_data = get_value_from_dict(data, data_key)
    tree = es_group2tree(group_data, [group_type])
    tree_with_percentage = covert2percentage(tree, "doc_count", flag)
    result = {
        "title": title,
        "data": [{"name": x.get("key"), "unit": unit, "value": x.get("percentage"),
                  "second_unit": second_unit, "second_value": x.get("doc_count")} for x in tree_with_percentage]
    }
    return result



import json
import random

import aiohttp
import nacos
import tornado.options
import requests
import py_eureka_client.eureka_client as eureka_client
from lesscode.web.status_code import StatusCode

from lesscode.web.business_exception import BusinessException


async def post(path, data=None,
               base_url=tornado.options.options.data_server,
               result_type="json", pack=False, **kwargs):
    project_name = tornado.options.options.project_name.encode("utf-8")
    flag = kwargs.pop("flag", True)
    if flag:
        if not kwargs.get("headers"):
            kwargs.update({"headers": {
                "Content-Type": "application/json",
                "Project-Name": project_name
            }})
        else:
            kwargs["headers"].update({"Project-Name": project_name})
    else:
        if not kwargs.get("headers"):
            kwargs.update({"headers": {
                "Project-Name": project_name
            }})
        else:
            kwargs["headers"].update({"Project-Name": project_name})
    async with aiohttp.ClientSession() as session:
        flag = kwargs.pop("flag", True)
        if flag:
            async with session.post(base_url + path, json=data, **kwargs) as resp:
                if result_type == "json":
                    result = await resp.json()
                    if not pack:
                        if result.get("status") == "00000":
                            result = result.get("data")
                        else:
                            message = f'ori_message:{result.get("status", "")}, {result.get("message", "未知错误")}'
                            raise BusinessException(StatusCode.COMMON_CODE_MESSAGE("C0001", message))
                elif result_type == "text":
                    result = await resp.text()
                else:
                    result = await resp.content.read()
                return result
        else:
            async with session.post(base_url + path, data=data, **kwargs) as resp:
                if result_type == "json":
                    result = await resp.json()
                    if not pack:
                        if result.get("status") == "00000":
                            result = result.get("data")
                        else:
                            message = f'ori_message:{result.get("status", "")}, {result.get("message", "未知错误")}'
                            raise BusinessException(StatusCode.COMMON_CODE_MESSAGE("C0001", message))
                elif result_type == "text":
                    result = await resp.text()
                else:
                    result = await resp.content.read()
                return result


async def get(path, params=None, base_url=tornado.options.options.data_server, result_type="json", pack=False,
              **kwargs):
    project_name = tornado.options.options.project_name.encode("utf-8")
    if not kwargs.get("headers"):
        kwargs.update({"headers": {
            "Project-Name": project_name
        }})
    else:
        kwargs["headers"].update({"Project-Name": project_name})

    async with aiohttp.ClientSession() as session:
        async with session.get(base_url + path, params=params, **kwargs) as resp:
            if result_type == "json":
                result = await resp.json()
                if not pack:
                    if result.get("status") == "00000":
                        result = result.get("data")
                    else:
                        message = f'ori_message:{result.get("status", "")}, {result.get("message", "未知错误")}'
                        raise BusinessException(StatusCode.COMMON_CODE_MESSAGE("C0001", message))
            elif result_type == "text":
                result = await resp.text()
            else:
                result = await resp.content.read()
            return result


def sync_get(path, params=None, base_url=None, result_type="json", pack=False, **kwargs):
    project_name = tornado.options.options.project_name.encode("utf-8")
    if not base_url:
        base_url = tornado.options.options.data_server
    if not kwargs.get("headers") and project_name:
        kwargs.update({"headers": {
            "Project-Name": project_name
        }})
    elif project_name:
        kwargs["headers"].update({"Project-Name": project_name})
    s = requests.session()
    try:
        res = s.get(base_url + path, params=params, **kwargs)
    except Exception as e:
        raise e
    finally:
        s.close()
    if result_type == "json":
        res = res.json()
        if not pack:
            if res.get("status") == "00000":
                res = res.get("data")
            else:
                message = f'ori_message:{res.get("status", "")}, {res.get("message", "未知错误")}'
                raise BusinessException(StatusCode.COMMON_CODE_MESSAGE("C0001", message))
    elif result_type == "text":
        res = res.text
    else:
        res = res.content
    return res


def sync_post(path, data=None,
              base_url=None,
              result_type="json", pack=False, **kwargs):
    project_name = tornado.options.options.project_name.encode("utf-8")
    if not base_url:
        base_url = tornado.options.options.data_server
    flag = kwargs.pop("flag", True)
    s = requests.session()
    try:
        if flag:
            if not kwargs.get("headers"):
                kwargs.update({"headers": {
                    "Content-Type": "application/json",
                    "Project-Name": project_name
                }})
            else:
                kwargs["headers"].update({"Project-Name": project_name})
            res = s.post(base_url + path, json=data, **kwargs)
        else:
            if not kwargs.get("headers"):
                kwargs.update({"headers": {
                    "Project-Name": project_name
                }})
            else:
                kwargs["headers"].update({"Project-Name": project_name})
            res = s.post(base_url + path, data=data, **kwargs)
    except Exception as e:
        raise e
    finally:
        s.close()
    if result_type == "json":
        res = res.json()
        if not pack:
            if res.get("status") == "00000":
                res = res.get("data")
            else:
                message = f'ori_message:{res.get("status", "")}, {res.get("message", "未知错误")}'
                raise BusinessException(StatusCode.COMMON_CODE_MESSAGE("C0001", message))
    elif result_type == "text":
        res = res.text
    else:
        res = res.content
    return res


def eureka_request(path="", return_type="json", method="GET", data=None, app_name="DATA_SERVICE", pack=False, **kwargs):
    flag = kwargs.pop("flag", True)
    if flag:
        if not kwargs.get("headers"):
            kwargs.update({"headers": {
                "Content-Type": "application/json"
            }})
    res = eureka_client.do_service(app_name=app_name, service=path, return_type=return_type, method=method, data=data,
                                   **kwargs)
    if return_type == "json" and not pack:
        if not pack:
            if res.get("status") == "00000":
                res = res.get("data")
            else:
                message = f'ori_message:{res.get("status", "")}, {res.get("message", "未知错误")}'
                raise BusinessException(StatusCode.COMMON_CODE_MESSAGE("C0001", message))
    return res


def eureka_get(path="", data=None, return_type="json", app_name="DATA_SERVICE", pack=False, **kwargs):
    base_url = tornado.options.options.data_server
    if kwargs.get("base_url") is not None:
        base_url = kwargs.pop("base_url")

    if tornado.options.options.running_env != "local":

        res = eureka_request(path=path, return_type=return_type, method="GET", data=data, app_name=app_name, pack=pack,
                             **kwargs)
    else:
        res = sync_get(path=path, params=data, base_url=base_url,
                       result_type=return_type,
                       pack=pack, **kwargs)
    return res


def eureka_post(path="", data=None, return_type="json", app_name="DATA_SERVICE", pack=False, **kwargs):
    base_url = tornado.options.options.data_server
    if kwargs.get("base_url") is not None:
        base_url = kwargs.pop("base_url")
    if tornado.options.options.running_env != "local":

        res = eureka_request(path=path, return_type=return_type, method="POST", data=json.dumps(data),
                             app_name=app_name,
                             pack=pack, **kwargs)
    else:
        res = sync_post(path=path, data=data, base_url=base_url,
                        result_type=return_type,
                        pack=pack, **kwargs)
    return res


def nacos_service_instance(service_name, namespace, clusters=None, group_name=None):
    server_addresses = tornado.options.options.nacos_config.get("server_addresses")
    client = nacos.NacosClient(server_addresses=server_addresses, namespace=namespace)
    service = client.list_naming_instance(service_name=service_name, clusters=clusters, group_name=group_name,
                                          healthy_only=True)
    service_hosts = service.get("hosts")
    if service_hosts:
        service_host = random.choice(service_hosts)
        return service_host
    else:
        message = f'ori_message:service_name={service_name} has no healthy instance'
        raise BusinessException(StatusCode.COMMON_CODE_MESSAGE("C0001", message))


def nacos_get(path, params=None, service_name=None, namespace="public", clusters=None, group_name=None,
              pack=False, return_type="json", **kwargs):
    base_url = tornado.options.options.data_server
    if kwargs.get("base_url") is not None:
        base_url = kwargs.pop("base_url")
    if tornado.options.options.running_env != "local":
        service_instance = nacos_service_instance(service_name=service_name,
                                                  namespace=namespace, clusters=clusters, group_name=group_name)
        base_url = f'http://{service_instance.get("ip")}:{service_instance.get("port")}'

    res = sync_get(path=path, params=params, base_url=base_url, result_type=return_type, pack=pack, **kwargs)
    return res


def nacos_post(path, data=None, service_name=None, namespace="public", clusters=None, group_name=None,
               pack=False, return_type="json", **kwargs):
    base_url = tornado.options.options.data_server
    if kwargs.get("base_url") is not None:
        base_url = kwargs.pop("base_url")
    if tornado.options.options.running_env != "local":
        service_instance = nacos_service_instance(service_name=service_name,
                                                  namespace=namespace, clusters=clusters, group_name=group_name)
        base_url = f'http://{service_instance.get("ip")}:{service_instance.get("port")}'

    res = sync_post(path=path, data=data, base_url=base_url, result_type=return_type, pack=pack, **kwargs)
    return res


def common_get(path, params=None, service_name=None, namespace="public", clusters=None, group_name=None,
               pack=False, return_type="json", **kwargs):
    base_url = tornado.options.options.data_server
    if kwargs.get("base_url") is not None:
        base_url = kwargs.pop("base_url")
    if tornado.options.options.request_type == "request":
        res = sync_get(path=path, params=params, base_url=base_url, result_type=return_type, pack=pack, **kwargs)
        return res
    elif tornado.options.options.request_type == "eureka":
        res = eureka_get(path=path, data=params, return_type=return_type, app_name=service_name, pack=pack, **kwargs)
        return res
    elif tornado.options.options.request_type == "nacos":
        res = nacos_get(path=path, params=params, service_name=service_name, namespace=namespace, clusters=clusters,
                        group_name=group_name, pack=pack, return_type=return_type, **kwargs)
        return res
    else:
        raise BusinessException(StatusCode.UNSUPPORTED_REQUEST_TYPE)


def common_post(path, data=None, service_name=None, namespace="public", clusters=None, group_name=None,
                pack=False, return_type="json", **kwargs):
    base_url = tornado.options.options.data_server
    if kwargs.get("base_url") is not None:
        base_url = kwargs.pop("base_url")
    if tornado.options.options.request_type == "request":
        res = sync_post(path=path, data=data, base_url=base_url, result_type=return_type, pack=pack, **kwargs)
        return res
    elif tornado.options.options.request_type == "eureka":
        res = eureka_post(path=path, data=data, return_type=return_type, app_name=service_name, pack=pack, **kwargs)
        return res
    elif tornado.options.options.request_type == "nacos":
        res = nacos_post(path=path, data=data, service_name=service_name, namespace=namespace, clusters=clusters,
                         group_name=group_name,
                         pack=pack, return_type=return_type, **kwargs)
        return res
    else:
        raise BusinessException(StatusCode.UNSUPPORTED_REQUEST_TYPE)



# -*- coding: utf-8 -*-
# @Time    : 2022/8/23 14:40
# @Author  : navysummer
# @Email   : navysummer@yeah.net
import logging
import traceback

import pika
from tornado.options import options


class RabbitMqHelper:
    def __init__(self, **kwargs):
        if kwargs.get("host"):
            host = kwargs.pop("host")
        else:
            host = options.rabbitmq_config.get("host", "127.0.0.1")
        if kwargs.get("port"):
            port = kwargs.pop("port")
        else:
            port = options.rabbitmq_config.get("port", 5672)
        if kwargs.get("username"):
            username = kwargs.pop("username")
        else:
            username = options.rabbitmq_config.get("username")
        if kwargs.get("password"):
            password = kwargs.pop("password")
        else:
            password = options.rabbitmq_config.get("password")
        if kwargs.get("queue"):
            self.queue = kwargs.pop("queue")
        else:
            raise Exception("missing queue")
        self.exchange = kwargs.pop("exchange", None)
        exchange_kwargs = kwargs.pop("exchange_kwargs", {})
        credentials = pika.PlainCredentials(username, password)
        self.connection = pika.BlockingConnection(
            pika.ConnectionParameters(host=host, port=port, credentials=credentials, **kwargs))
        self.channel = self.connection.channel()
        queue_kwargs = kwargs.get("queue_kwargs", {})
        self.channel.queue_declare(queue=self.queue, durable=True, **queue_kwargs)
        if self.exchange:
            self.channel.exchange_declare(exchange=self.exchange, **exchange_kwargs)

    def publish(self, message: str, exchange="", **kwargs):
        if isinstance(message, str):
            message = message.encode("utf-8")
        elif not isinstance(message, bytes):
            raise Exception("message must be str or bytes")
        res = self.channel.basic_publish(exchange=exchange, routing_key=self.queue, body=message,
                                         properties=pika.BasicProperties(
                                             delivery_mode=2
                                         ), **kwargs)
        logging.info(f"send message={message.decode('utf-8')} to queue={self.queue}")
        self.connection.close()
        data = {"message": message, "queue": self.queue, "res": res}
        return data

    def consume(self, callback, **kwargs):
        logging.info("Waiting for messages")
        self.channel.basic_consume(queue=self.queue, on_message_callback=callback, **kwargs)
        try:
            self.channel.start_consuming()
        except Exception:
            logging.error(traceback.format_exc())



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2022/2/17 2:45 下午
# Copyright (C) 2022 The lesscode Team
import ssl

from nebula3.gclient.net import ConnectionPool
from nebula3.Config import Config, SSL_config

from lesscode.db.base_connection_pool import BaseConnectionPool


class NebulaPool(BaseConnectionPool):
    """
    mysql 数据库链接创建类
    """

    async def create_pool(self):
        pass

    def sync_create_pool(self):
        config = Config()
        ssl_conf = None
        config.max_connection_pool_size = self.conn_info.max_size
        config.min_connection_pool_size = self.conn_info.min_size
        if self.conn_info.params and isinstance(self.conn_info.params, dict):
            config.timeout = self.conn_info.params.get("timeout", 0)
            config.idle_time = self.conn_info.params.get("idle_time", 0)
            config.interval_check = self.conn_info.params.get("interval_check", -1)
            ssl_config = self.conn_info.params.get("ssl_conf", {})
            if ssl_conf and isinstance(ssl_conf, dict):
                ssl_conf = SSL_config()
                ssl_conf.unix_socket = ssl_config.get("unix_socket", None)
                ssl_conf.ssl_version = ssl_config.get("ssl_version", None)
                ssl_conf.cert_reqs = ssl_config.get("cert_reqs", ssl.CERT_NONE)
                ssl_conf.ca_certs = ssl_config.get("ca_certs", None)
                ssl_conf.verify_name = ssl_config.get("verify_name", None)
                ssl_conf.keyfile = ssl_config.get("keyfile", None)
                ssl_conf.certfile = ssl_config.get("certfile", None)
                ssl_conf.allow_weak_ssl_versions = ssl_config.get("allow_weak_ssl_versions", None)
        pool = ConnectionPool()
        pool.init([(self.conn_info.host, self.conn_info.port)], config, ssl_conf)
        return pool



import functools
import json
import uuid

import tornado.options

from lesscode.web.business_exception import BusinessException
from lesscode.web.status_code import StatusCode


class User:
    """
    用户对象类
    """

    def __init__(self):
        # '账号id',
        self.id = None
        # 用户名
        self.username = None
        # '密码',
        self.password = None
        # '显示名',
        self.display_name = None
        # 手机号,
        self.phone_no = None
        #  邮箱
        self.email = None
        # 组织机构id',
        self.org_id = None
        # '1正常（激活）；2未激活（管理员新增，首次登录需要改密码）； 3锁定（登录错误次数超限，锁定时长可配置）； 4休眠（长期未登录（字段，时长可配置），定时） 5禁用-账号失效；
        self.account_status = None
        # 角色id集合
        self.roleIds = None


def user_verification(username="admin", **kw):
    def wrapper(func):
        @functools.wraps(func)
        def run(self, *args, **kwargs):
            if tornado.options.options.running_env != "local":
                user_str = self.request.headers.get("User")
                if user_str:
                    user = json.loads(user_str)
                    if isinstance(user, dict):
                        user_username = user.get("username")
                        if username != user_username:
                            raise BusinessException(StatusCode.ACCESS_DENIED)
                    else:
                        raise BusinessException(StatusCode.ACCESS_DENIED)
                else:
                    raise BusinessException(StatusCode.ACCESS_DENIED)

            return func(self, *args, **kwargs)

        return run

    return wrapper


def login_verification(func):
    @functools.wraps(func)
    def wrapper(self, *args, **kwargs):
        if tornado.options.options.running_env != "local":
            user = self.request.headers.get("User")
            if not user:
                raise BusinessException(StatusCode.INVALID_TOKEN)
            else:
                user_info = json.loads(user)
                if not user_info:
                    raise BusinessException(StatusCode.INVALID_TOKEN)
        return func(self, *args, **kwargs)

    return wrapper



# -*- coding: utf-8 -*-
import json
import traceback
from datetime import datetime
from decimal import Decimal
from typing import Optional, Awaitable, Union

from bson import ObjectId
from tornado.web import RequestHandler

from lesscode.utils.es_log.record_log import es_record_log
from lesscode.web.business_exception import BusinessException
from lesscode.web.response_result import ResponseResult
from lesscode.web.status_code import StatusCode


class NativeHandler(RequestHandler):
    def prepare(self):
        self.process_request(self.request)

    def process_request(self, request):
        """
        预处理请求中间件，对请求内容进行改写
        :param request:
        :return:
        """
        pass

    def process_view(self, request, view_func, view_args, view_kwargs):
        """
        视图中间件，对视图进行处理
        :param request:
        :param view_func:
        :param view_args:
        :param view_kwargs:
        :return:
        """
        pass

    def process_exception(self, exception: Exception):
        """
        异常处理中间件
        :param exception:
        :return:
        """
        if isinstance(exception, BusinessException):
            self.write(json.dumps(ResponseResult(status_code=exception.status_code), ensure_ascii=False))
        else:
            self.write(json.dumps(ResponseResult(StatusCode.INTERNAL_SERVER_ERROR(exception)), ensure_ascii=False))

    def process_response(self, data):
        """
        处理响应数据的中间件
        :param data:
        :return:
        """
        self.write(data)

    def write_error(self, status_code, **kwargs):
        es_record_log(request=self.request, message=traceback.format_exc(), level="error", status_code=status_code)
        self.process_exception(exception=kwargs.get('exc_info')[1])

    def write(self, chunk: Union[str, bytes, dict] = None):
        super(NativeHandler, self).write(chunk)
        self.finish()

    def json_response(self, data):
        if not isinstance(data, str) and not isinstance(data, bytes) and not isinstance(data, dict):
            data = json.dumps(ResponseResult(data=data), ensure_ascii=False, cls=JSONEncoder)
        self.process_response(data)

    def response(self, data):
        self.process_response(data)

    def data_received(self, chunk: bytes) -> Optional[Awaitable[None]]:
        pass

    def post(self):
        pass

    def get(self):
        pass

    def put(self):
        pass

    def delete(self):
        pass


class JSONEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, ObjectId):
            return str(o)
        elif isinstance(o, datetime):
            return str(o)
        elif isinstance(o, bytes):
            return str(o, encoding="utf-8")
        elif isinstance(o, set):
            return list(o)
        elif isinstance(o, Decimal):
            return str(o)
        return json.JSONEncoder.default(self, o)



# -*- coding: utf-8 -*-
# @Time    : 2022/8/24 10:10
# @Author  : navysummer
# @Email   : navysummer@yeah.net
import copy
import json
import logging
import traceback

import kafka
from kafka import KafkaConsumer, KafkaProducer
from tornado.options import options


class KafkaHelper:
    def __init__(self, **kwargs):
        if kwargs.get("bootstrap_servers"):
            self.bootstrap_servers = kwargs.pop("bootstrap_servers")
        else:
            self.bootstrap_servers = options.kafka_config.get("bootstrap_servers", ["127.0.0.1:9092"])
        if kwargs.get("topic"):
            self.topic = kwargs.pop("topic")
        else:
            raise Exception("missing topic")
        #
        self.sasl_config = {}
        if kwargs.get("username"):
            username = kwargs.pop("username")
        else:
            username = options.kafka_config.get("username")
        if kwargs.get("password"):
            password = kwargs.pop("password")
        else:
            password = options.kafka_config.get("password")
        if username and password:
            self.sasl_config = {
                "sasl_mechanism": "PLAIN",
                "security_protocol": "SASL_PLAINTEXT",
                "sasl_plain_username": username,
                "sasl_plain_password": password
            }
        self.api_version = tuple([int(x) for x in kafka.__version__.split(".")])

    def publish(self, value=None, key=None, headers=None, partition=None, timestamp_ms=None, **configs):
        producer = KafkaProducer(bootstrap_servers=self.bootstrap_servers,
                                 api_version=self.api_version,
                                 **self.sasl_config,
                                 **configs)
        try:
            value = json.dumps(value).encode("utf-8")
            key = json.dumps(key).encode("utf-8")
            future = producer.send(self.topic, value=value, key=key, headers=headers, partition=partition,
                                   timestamp_ms=timestamp_ms)
            record_metadata = future.get(timeout=10)
            data = {
                "topic": self.topic,
                "value": value,
                "key": key,
                "partition": partition if partition else record_metadata.partition,
                "offset": record_metadata.offset
            }
            logging.info(f'send data={data}')
        except Exception:
            logging.error(traceback.format_exc())

    def consume(self, callback, **configs):
        consumer = KafkaConsumer(self.topic, bootstrap_servers=self.bootstrap_servers,
                                 api_version=self.api_version,
                                 enable_auto_commit=False,
                                 **self.sasl_config, **configs)
        callback(consumer=consumer)



import logging
import traceback
from decimal import Decimal

from sqlalchemy.orm import sessionmaker, scoped_session
from sqlalchemy.util.compat import contextmanager
from tornado.options import options


class SqlAlchemyHelper:
    def __init__(self, pool):
        """
        初始化sql工具
        :param pool: 连接池名称
        """
        if isinstance(pool, str):
            self.pool, self.conn_info = options.database[pool]
        else:
            self.pool = pool

    @contextmanager
    def make_session(self, **kwargs):
        db_session = scoped_session(sessionmaker(bind=self.pool, **kwargs))
        session = db_session()
        yield session
        session.commit()
        session.close()


def alchemy_default_to_dict(params, data, repetition=False):
    data_list = []
    key_list = []
    if repetition:
        for arg in params:
            if arg.key:
                key_list.append(arg.key)
            else:
                key_list.append(arg.name)
    else:
        for arg in params:
            arg = str(arg)
            if "(" in arg and ")" in arg:
                key_list.append(arg.split(".")[-1][:-1])
            else:
                key_list.append(arg.split(".")[-1])
    if isinstance(data, list):
        for d in data:
            dict_data = dict(zip(key_list, d))
            data_list.append(dict_data)
        return data_list
    else:
        if data:
            return dict(zip(key_list, data))
        else:
            return {}


def sqlalchemy_paging(Query, limit_number, offset_number):
    data_list = Query.limit(limit_number).offset(offset_number).all()
    data_count = Query.count()
    return {"count": data_count, "dataSource": data_list}


def query_set_to_dict(obj):
    if hasattr(obj, "__table__"):
        obj_dict = {}
        for column in obj.__table__.columns.keys():
            if hasattr(obj, column):
                val = getattr(obj, column)
                if isinstance(val, Decimal):
                    val = float(val)
                obj_dict[column] = val
        return obj_dict
    elif hasattr(obj, "keys"):
        return {key: getattr(obj, key) for key in obj.keys()}
    else:
        return dict(obj)


def query_set_to_list(query_set):
    ret_list = []
    for obj in query_set:
        ret_dict = query_set_to_dict(obj)
        ret_list.append(ret_dict)
    return ret_list


def result_to_json(data):
    if isinstance(data, list):
        return query_set_to_list(data)
    else:
        return query_set_to_dict(data)


def result_page(query, page_num=1, page_size=10):
    offset_number = (page_num - 1) * page_size if page_num >= 1 else 0
    data_list = result_to_json(query.limit(page_size).offset(offset_number).all())
    data_count = query.count()
    return {"count": data_count, "dataSource": data_list}



import json
import logging

from tornado.web import RequestHandler

from lesscode.utils.json import JSONEncoder
from lesscode.web.response_result import ResponseResult
from lesscode.web.status_code import StatusCode


class NotFoundHandler(RequestHandler):
    def set_default_headers(self):
        self.set_header("Access-Control-Allow-Origin", "*")
        self.set_header("Request-Id", self.request.headers.get("Request-Id", ""))
        self.set_header("Plugin-Running-Time", self.request.headers.get("Plugin-Running-Time", ""))
        self.set_header("Access-Control-Allow-Headers",
                        "x-requested-with,Authorization,Can-read-cache,Content-Type,User")
        self.set_header("Access-Control-Allow-Methods", "POST,GET,PUT,DELETE,OPTIONS")
        self.set_header("Access-Control-Expose-Headers", "Access-Token")
        self.set_header("Access-Control-Allow-Credentials", "true")
        self.set_header("Content-Type", "application/json; charset=UTF-8")

    def data_received(self, chunk: bytes):
        pass

    async def get(self):
        self.write(json.dumps(ResponseResult(StatusCode.REQUEST_PATH_NOT_FOUND), ensure_ascii=False, cls=JSONEncoder))

    async def post(self):
        self.write(json.dumps(ResponseResult(StatusCode.REQUEST_PATH_NOT_FOUND), ensure_ascii=False, cls=JSONEncoder))

    async def put(self):
        self.write(json.dumps(ResponseResult(StatusCode.REQUEST_PATH_NOT_FOUND), ensure_ascii=False, cls=JSONEncoder))

    async def patch(self):
        self.write(json.dumps(ResponseResult(StatusCode.REQUEST_PATH_NOT_FOUND), ensure_ascii=False, cls=JSONEncoder))

    async def delete(self):
        self.write(json.dumps(ResponseResult(StatusCode.REQUEST_PATH_NOT_FOUND), ensure_ascii=False, cls=JSONEncoder))

    async def head(self):
        self.write(json.dumps(ResponseResult(StatusCode.REQUEST_PATH_NOT_FOUND), ensure_ascii=False, cls=JSONEncoder))

    async def options(self):
        self.write(json.dumps(ResponseResult(StatusCode.REQUEST_PATH_NOT_FOUND), ensure_ascii=False, cls=JSONEncoder))



# -*- coding: utf-8 -*-
# @Time    : 2022/11/7 16:38
# @Author  : navysummer
# @Email   : navysummer@yeah.net
import datetime
import json
import logging
import traceback

import requests
from requests.auth import HTTPBasicAuth
from tornado.options import options

from lesscode.db.ds_helper import DSHelper
from lesscode.utils.json import JSONEncoder


def es_record_log(request, message="", level="info", status_code=200, task_id=None):
    log_es_config = options.log_es_config if options.log_es_config else {}
    if log_es_config.get("enable"):
        request_headers = dict(request.headers)
        request_params = dict(request.query_arguments)
        content_type = request.headers.get('Content-Type')
        request_body = request.body
        request_id = request_headers.get("Request-Id", "")
        log_data = {
            "timestamp": datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
            "task_id": task_id,
            "level": level,
            "request": {
                "id": request_id,
                "header": request_headers,
                "params": request_params,
                "url": request.path,
                "Content-Type": content_type,
                "method": request.method,
                "body": request_body,
                "real_ip": request.remote_ip
            },
            "status_code": status_code,
            "message": message
        }
        connect_name = log_es_config.get("connect_name")
        protocol = log_es_config.get('protocol', 'http')
        es_index = log_es_config.get('index', "request_log")
        logging.info(f"es_record_log task task_id={task_id}")
        send_es(protocol, connect_name, es_index, log_data)


def send_es(protocol, connect_name, es_index, body):
    try:
        pool, conn_info = options.database[connect_name]
        url = f"{protocol}://{conn_info.host}:{conn_info.port}/{es_index}/_doc/"
        res = requests.post(
            url,
            data=json.dumps(body, ensure_ascii=False, cls=JSONEncoder).encode('utf-8'),
            headers={'content-type': "application/json"},
            auth=HTTPBasicAuth(conn_info.user, conn_info.password)
        )

        if res.ok:
            logging.info(f"send es url={url} success")
        else:
            logging.info(f"send es url={url} fail")
    except Exception as e:
        logging.error(traceback.format_exc())



from datetime import datetime, timedelta

import tornado.options
from bson import ObjectId
from lesscode.utils.encryption_algorithm import AES


def condition_by_like(find_condition, column, value):
    if value:
        find_condition[column] = {"$regex": f".*{value}.*"}


def condition_by_in(find_condition, column, param_list, is_object=False, _include=True, is_need_decrypt_oralce=False):
    if param_list:
        param_list = [i for i in param_list if i not in [None, "", "all"]]
        if len(param_list) > 0:
            if is_object:
                param_list = [ObjectId(id) for id in param_list]
            if is_need_decrypt_oralce:
                try:
                    param_list = [int(AES.decrypt(tornado.options.options.aes_key, id)) for id in param_list]
                except:
                    pass
            if _include is True:
                find_condition[column] = {"$in": param_list}
            else:
                find_condition[column] = {"$ne": {"$in": param_list}}


def condition_by_between(column, data_list, and_condition_list, is_contain_end=True, is_contain_start=True):
    if data_list:
        if data_list[0]:
            if is_contain_start:
                and_condition_list.append({column: {"$gte": data_list[0]}})
            else:
                and_condition_list.append({column: {"$gt": data_list[0]}})
        if len(data_list) == 2 and data_list[1]:
            if is_contain_end:
                and_condition_list.append({column: {"$lte": data_list[1]}})
            else:
                and_condition_list.append({column: {"$lt": data_list[1]}})


def condition_by_eq(find_condition, column, param, _include=True):
    if param is not None and param != "":
        if _include is True:
            find_condition[column] = param
        else:
            find_condition[column] = {"$ne": param}


def mongodb_sql_paging(page_size, page_num):
    limit = int(page_size)
    skip = (int(page_num) - 1) * int(page_size)

    return limit, skip



#!/usr/bin/env python3
import argparse
import os
from pathlib import Path

parser = argparse.ArgumentParser()
group = parser.add_mutually_exclusive_group()
group.add_argument("-c", "--project", help="create new project in current dir")
group.add_argument("-i", "--is_current_dir", help="current dir as project dir", type=bool, default=True)
args = parser.parse_args()

config_text = """# -*- coding: utf-8 -*-
from tornado.options import options
from lesscode.db.connection_info import ConnectionInfo

# 运行环境
options.running_env = "local"
# 项目前缀
options.route_prefix = ""
# 服务启动端口
options.port = 8000

# 日志级别
# 10-DEBUG       输出详细的运行情况，主要用于调试。
# 20-INFO        确认一切按预期运行，一般用于输出重要运行情况。
# 30-WARNING     系统运行时出现未知的事情（如：警告内存空间不足），但是软件还可以继续运行，可能以后运行时会出现问题。
# 40-ERROR       系统运行时发生了错误，但是还可以继续运行。
# 50-CRITICAL    一个严重的错误，表明程序本身可能无法继续运行。
options.logging = "DEBUG"
# 日志文件分割方式，时间与文件大小，默认采用时间分割
# time/size
options.log_rotate_mode = "time"
# 日志文件名前缀
options.log_file_prefix = "log"
# 日志文件间隔的时间单位
# S 秒
# M 分
# H 小时、
# D 天、
# W 每星期（interval==0时代表星期一）
# midnight 每天凌晨
options.log_rotate_when = "D"
# 备份文件的个数，如果超过这个个数，就会自动删除
options.log_file_num_backups = 30

# rabbitmq配置
options.rabbitmq_config = {
    "host": "127.0.0.1",
    "port": 5672,
    "username": "guest",
    "password": "guest"
}

# kafka配置
options.kafka_config = {
    "bootstrap_servers": ["120.92.35.156:8985"]
}

# 金山对象存储配置
options.ks3_connect_config = {"host": "ks3-cn-beijing.ksyun.com", "access_key_id": "123456",
                              "access_key_secret": "123456"}

# 任务调度配置                            
options.scheduler_config = {
    "enable": True
}

# 是否打印sql
options.echo_sql = True

# 数据库连接配置
options.conn_info = [
    ConnectionInfo(dialect="postgresql", host="127.0.0.1", port=5432, user="root", password="root",
                   db_name="test", enable=True),
    ConnectionInfo(dialect="mongodb", name="mongodb", host="127.0.0.1", port=27017, user="root",
                   password="root", enable=True),
    ConnectionInfo(dialect="mysql", name="mysql", host="127.0.0.1", port=3306, user="root",
                   password="root", db_name="test", enable=True),
    ConnectionInfo(dialect="sqlalchemy", name="sa", host="127.0.0.1", port=3306, user="root",
                   password="root", db_name="test", params={"db_type":"mysql"}, enable=True),
    ConnectionInfo(dialect="elasticsearch", name="es", host="127.0.0.1", port=9200, user="root",
                   password="root", enable=True),
    ConnectionInfo(dialect="esapi", name="esapi", host="127.0.0.1", port=9200, user="root",
                   password="root", enable=True),
    ConnectionInfo(dialect="neo4j", name="neo4j", host="127.0.0.1", port=7474, user="neo4j",
                   password="neo4j", db_name="neo4j", enable=True),
    ConnectionInfo(dialect="redis", name="redis", host="localhost", port=6379, user=None,
                   password=None, db_name=1, enable=True)
]

"""

requirements_text = """lesscode-py>=0.2.60"""
README_text = f"""# {args.project}
## 框架涉及的包：
```
  "tornado==6.0",
  "tornado-sqlalchemy==0.7.0",
  "aiomysql==0.0.22",
  "motor==2.5.1",
  "elasticsearch==7.15.2",
  "aiohttp==3.8.1",
  "crypto==1.4.1",
  "pycryptodome==3.12.0",
  "aioredis==2.0.1",
  "DBUtils==3.0.2",
  "redis==4.1.4",
  "requests==2.27.1",
  "neo4j==5.0.0",
  "snowland-smx==0.3.1",
  "py_eureka_client==0.11.3",
  "ks3sdk==1.5.0",
  "filechunkio==1.8",
  "APScheduler==3.9.1",
  "nacos-sdk-python==0.1.8",
  "pika==1.3.0",
  "kafka-python==2.0.2"
  "aiopg>=1.3.3"  # 除了pg库没安装，以上包均已安装，如有需要自行安装
```
  
"""
server_text = """# -*- coding: utf-8 -*-
from lesscode.web.web_server import WebServer

if __name__ == "__main__":
    server = WebServer()
    server.start()
"""

demo_handler_file_text = """# -*- coding: utf-8 -*-
from sqlalchemy import NVARCHAR, Column
#from sqlalchemy.orm import declarative_base
from sqlalchemy.ext.declarative import declarative_base
from lesscode.db.sqlalchemy.sqlalchemy_helper import SqlAlchemyHelper, result_to_json, result_page
from lesscode.web.base_handler import BaseHandler
from lesscode.web.router_mapping import Handler, GetMapping, PostMapping

Base = declarative_base()


class LcAuthUser(Base):
    __tablename__ = "lc_auth_user"
    id = Column(NVARCHAR, primary_key=True)
    username = Column()
    account_status = Column()
    password = Column()
    phone_no = Column()
    display_name = Column()
    create_time = Column()


@Handler("/sqlalchemy")
class SqlAlchemyHandler(BaseHandler):

    @PostMapping("/test")
    def test(self):
        with SqlAlchemyHelper("auth_engine").make_session() as session:
            params = [LcAuthUser.id, LcAuthUser.username, LcAuthUser.phone_no]
            al = session.query(*params).all()
            return result_to_json(al)
"""


def create_file_or_dir(path, path_type, text=""):
    path_obj = Path(path)
    if not path_obj.exists():
        if path_type == 0:
            os.mkdir(path)
        elif path_type == 1:
            with open(path, 'w+') as file:
                file.write(text)
    if not path_obj.exists():
        if path_type == 0:
            path_type = "目录"
        elif path_type == 1:
            path_type = "文件"
        raise Exception(f"{path_type}({path})创建失败")


def main():
    current_dir = os.getcwd()
    if args.project:
        project_dir = f"{current_dir}/{args.project}"
    else:
        project_dir = current_dir
    handlers_dir = f"{project_dir}/handlers"
    profiles_dir = f"{project_dir}/profiles"
    create_file_or_dir(project_dir, path_type=0)
    create_file_or_dir(handlers_dir, path_type=0)
    create_file_or_dir(profiles_dir, path_type=0)

    demo_handler_file = f"{handlers_dir}/demo_handler.py"
    create_file_or_dir(demo_handler_file, path_type=1, text=demo_handler_file_text)
    config_file = f"{profiles_dir}/config.py"
    create_file_or_dir(config_file, path_type=1, text=config_text)
    requirements_file = f"{project_dir}/requirements.txt"
    create_file_or_dir(requirements_file, path_type=1, text=requirements_text)
    reade_me_file = f"{project_dir}/README.md"
    create_file_or_dir(reade_me_file, path_type=1, text=README_text)
    server_file = f"{project_dir}/server.py"
    create_file_or_dir(server_file, path_type=1, text=server_text)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/11/11 8:06 下午
# Copyright (C) 2021 The lesscode Team
import asyncio
import os
import sys
import logging
import traceback

import tornado.web
import tornado.ioloop
import tornado.httpserver
from apscheduler.schedulers.tornado import TornadoScheduler
from tornado.log import LogFormatter as TornadoLogFormatter  # 避免命名歧义，设置了别名
from tornado.options import options, define
from typing import Optional

from tornado_sqlalchemy import SQLAlchemy

from lesscode.db.init_connection_pool import InitConnectionPool
from lesscode.extend_handlers.not_found_handler import NotFoundHandler
from lesscode.web.router_mapping import RouterMapping
import platform

define("application_name", default=f"{os.path.split(os.path.dirname(sys.argv[0]))[-1]}", type=str, help="配置文件")
define("project_name", default=f"{os.path.split(os.path.dirname(sys.argv[0]))[-1]}", type=str, help="配置文件")
define("application_path", default=f"{os.path.abspath(os.path.dirname(sys.argv[0]))}", type=str, help="应用运行根路径")
define("cookie_secret", default="hRHQlJoUQIiNai2MLV9fQV3YDpxWrUHQp/jnfF9riQk=", type=str, help="cookie 秘钥 ")
define("static_path", default=os.path.join(f"{os.path.abspath(os.path.dirname(sys.argv[0]))}", "static"), type=str,
       help="静态资源目录")
define("aes_key", default="haohaoxuexi", type=str, help="aes加密key")
define("data_server", default="http://127.0.0.1:8901", type=str, help="数据服务")
define("profile", default="profiles.config", type=str, help="配置文件")
define("port", default="8080", type=int, help="服务监听端口")
define("handler_path", default="handlers", type=str, help="处理器文件存储路径")
define("echo_sql", default=True, type=bool, help="是否启用SQL输出")
define("async_enable", default=False, type=bool, help="是否启用异步")
define("max_limit", default=500, type=int, help="单次查询最大数量")
define("eureka_config", default={}, type=dict, help="eureka配置")
define("rms_register_enable", default=False, type=bool, help="是否启动资源注册")
define("rms_register_server", default="http://127.0.0.1:8918", type=str, help="启动资源注册")
define("running_env", default="local", type=str, help="运行环境")
define("request_type", default="request", type=str, help="请求类型")
define("db", default=SQLAlchemy(), type=SQLAlchemy, help="SqlAlchemy")
define("ks3_connect_config", default={}, type=dict, help="金山对象存储连接配置")
define("scheduler", default=None, type=TornadoScheduler, help="Tornado任务调度")
define("scheduler_config", default={}, type=dict, help="Tornado任务调度配置")
define("nacos_config", default={}, type=dict, help="nacos配置")
define("restful_handler", default=[], type=list, help="restful handler")
define("rabbitmq_config", default={}, type=dict, help="rabbitmq配置")
define("kafka_config", default={}, type=dict, help="kafka配置")
define("log_es_config", default={}, type=dict, help="同步日志到es的配置")
define("cache_conn", default="redis_con_1", type=str, help="redis缓存连接")
define("cache_enable", default=False, type=bool, help="redis缓存开关")
define("task_list", default=[], type=list, help="任务列表")
define("correction_param", default={"enable": False}, type=dict, help="aes加密key")
# 日志初始化配置，配置文件中可修改设置
options.logging = "DEBUG"
options.log_rotate_mode = "time"
options.log_file_prefix = "log"
options.log_rotate_when = "D"
options.log_file_num_backups = 30

options.running_env = "local"


class LogFormatter(TornadoLogFormatter):
    """
    LogFormatter 类用于日志统一格式化处理
    """

    def __init__(self):
        super(LogFormatter, self).__init__(
            fmt='%(color)s[%(asctime)s] [%(levelname)s] [%(module)s:%(lineno)d]%(end_color)s [%(message)s]',
            datefmt='%Y-%m-%d %H:%M:%S')


# 应用实例
def init_log():
    """
    初始化日志配置，创建控制台日志输出处理类，统一设置格式
    :return:
    """
    console_log = logging.StreamHandler()  # 创建控制台日志输出处理类
    console_log.setLevel(options.logging)  # 设置日志级别
    logging.getLogger().handlers.append(console_log)
    # 为日志处理类指定统一的日志输出格式
    [handler.setFormatter(LogFormatter()) for handler in logging.getLogger().handlers]


def load_profile():
    """
    初始化配置文件，配置文件可通过命令行参数指定 --profile=dev
    :return:
    """
    command_profile_flag = False  # 标识配置文件是否通过命令行参数指定
    # 获取命令行参数，此处仅获取profile配置信息
    profile = [item for item in sys.argv if item.__contains__("--profile=")]
    if profile:  # 如果有指定配置文件，更新参数，如果指定多次，取最后一次
        options.profile = f"profiles.config_{profile[-1].replace('--profile=', '')}"
        command_profile_flag = True
    # 第一次加载配置文件
    try:
        __import__(options.profile)
        # command_profile_flag  是False 表示加载的是默认配置文件，还需要再一次判断默认文件中是否指定了其他配置文件，此判断仅针对默认配置文件，其他配置文件不在进行判断。
        if not command_profile_flag and options.profile != "profiles.config":
            __import__(f"profiles.config_{options.profile}")
    except ModuleNotFoundError:
        # 如果加载失败直接跳过，继续执行，给予WARNING。
        logging.warning(f"配置文件不存在：{options.profile}")


class WebServer:
    """
    Wbe服务对象
    """

    def __init__(self):
        load_profile()  # 1、优先处理加载配置文件，要在命令行解析参数前
        tornado.options.parse_command_line()  # 2、调用tornado的命令行参数解析
        init_log()  # 3、初始化日志配置，要在命令行解析参数后，确保日志参数都已经设置完成。
        # 4、启动时创建数据库连接池
        if options.async_enable:
            tornado.ioloop.IOLoop.current().run_sync(InitConnectionPool.create_pool)
        else:
            InitConnectionPool.sync_create_pool()
        # 5、自动化注册Handler处理类，
        self.autoRegisterHandler(f"{os.path.abspath(os.path.dirname(sys.argv[0]))}/{options.handler_path}",
                                 options.handler_path)
        self.autoRegisterExtendHandler()
        if options.scheduler_config.get("enable", False):
            self.init_scheduler()
        handlers = options.restful_handler + RouterMapping.instance()
        handlers.append(('.*', NotFoundHandler))
        self.app = tornado.web.Application(
            handlers,
            cookie_secret=options.cookie_secret,
            static_path=options.static_path,
            db=options.db,
            debug=True if options.logging == "DEBUG" else False  # 依据日志记录级别确认是否开启调试模式
        )
        self.server = tornado.httpserver.HTTPServer(self.app, max_buffer_size=500 * 1024 * 1024, xheaders=True)  # 500M

    def start(self, num_processes: Optional[int] = 1, max_restarts: int = None):
        if platform.system() == "Windows":
            asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
        self.service_register()
        self.server.listen(options.port)
        self.server.start(num_processes, max_restarts)
        logging.info(f"start server : {options.application_name}")
        logging.info(f"start server port : {options.port}")
        try:
            tornado.ioloop.IOLoop.current().start()
        except KeyboardInterrupt:
            pass
        finally:
            # 关掉连接池
            for item in options.database.items():
                if hasattr(item[1][0], "close"):
                    tornado.ioloop.IOLoop.current().run_sync(item[1][0].close)
        return self

    def autoRegisterHandler(self, path, pkg_name):
        """
        动态注册Handler模块
        遍历项目指定包内的Handler，将包内module引入。
        :param path: 项目内Handler的文件路径
        :param pkg_name: 引入模块前缀
        """
        # 首先获取当前目录所有文件及文件夹
        dynamic_handler_names = os.listdir(path)
        for handler_name in dynamic_handler_names:
            # 利用os.path.join()方法获取完整路径
            full_file = os.path.join(path, handler_name)
            # 循环判断每个元素是文件夹还是文件
            if os.path.isdir(full_file):
                # 文件夹递归遍历
                self.autoRegisterHandler(os.path.join(path, handler_name), ".".join([pkg_name, handler_name]))
            elif os.path.isfile(full_file) and handler_name.lower().endswith("handler.py"):
                # 文件，并且为handler结尾，认为是请求处理器，完成动态装载
                ft = "{}.{}".format(pkg_name, handler_name.replace(".py", ""))
                __import__("{}.{}".format(pkg_name, handler_name.replace(".py", "")))

    def autoRegisterExtendHandler(self):
        # 首先获取当前目录所有文件及文件夹
        import lesscode
        path = f"{os.path.abspath(os.path.dirname(lesscode.__file__))}/extend_handlers"
        pkg_name = "lesscode.extend_handlers"
        dynamic_handler_names = os.listdir(path)
        for handler_name in dynamic_handler_names:
            # 利用os.path.join()方法获取完整路径
            full_file = os.path.join(path, handler_name)
            # 循环判断每个元素是文件夹还是文件
            if os.path.isdir(full_file):
                # 文件夹递归遍历
                self.autoRegisterHandler(os.path.join(path, handler_name), ".".join([pkg_name, handler_name]))
            elif os.path.isfile(full_file) and handler_name.lower().endswith("handler.py"):
                # 文件，并且为handler结尾，认为是请求处理器，完成动态装载
                __import__("{}.{}".format(pkg_name, handler_name.replace(".py", "")))

    def init_scheduler(self):
        options.scheduler = TornadoScheduler(gconfig=options.scheduler_config.get("config", {}),
                                             **options.scheduler_config.get("options", {}))
        options.scheduler.start()
        logging.info('[Scheduler Init]APScheduler has been started')
        if options.scheduler_config.get("enable") and options.task_list:
            for task in options.task_list:
                if isinstance(task, dict):
                    task_func = task.get("func")
                    task_kwargs = task.get("kwargs", {})
                    if task_func:
                        options.scheduler.add_job(func=task_func, **task_kwargs)

    def service_register(self):
        try:
            if options.eureka_config.get("enable"):
                import py_eureka_client.eureka_client as eureka_client
                eureka_instance_name = options.eureka_config.get("instance_name") if options.eureka_config.get(
                    "instance_name") else options.application_name
                eureka_instance_ip = options.eureka_config.get("instance_ip") if options.eureka_config.get(
                    "instance_ip") else "127.0.0.1"
                eureka_instance_port = options.eureka_config.get("instance_port") if options.eureka_config.get(
                    "instance_port") else options.port
                eureka_client.init(eureka_server=options.eureka_config.get("eureka_server"),
                                   app_name=eureka_instance_name,
                                   instance_ip=eureka_instance_ip,
                                   instance_port=eureka_instance_port)
                logging.info(f'service={eureka_instance_name} register success')
            if options.nacos_config.get("enable"):
                import nacos
                nacos_service_name = options.nacos_config.get("service_name") if options.nacos_config.get(
                    "service_name") else options.application_name
                nacos_namespace = options.nacos_config.get("namespace") if options.nacos_config.get(
                    "namespace") else "public"
                nacos_instance_ip = options.nacos_config.get("instance_ip") if options.nacos_config.get(
                    "instance_ip") else "127.0.0.1"
                nacos_instance_port = options.nacos_config.get("instance_port") if options.nacos_config.get(
                    "instance_port") else options.port
                nacos_instance_weight = options.nacos_config.get("instance_weight") if options.nacos_config.get(
                    "instance_weight") else 1.0
                nacos_instance_enable = options.nacos_config.get("instance_enable") if options.nacos_config.get(
                    "instance_enable") else True
                nacos_instance_healthy = options.nacos_config.get("instance_healthy") if options.nacos_config.get(
                    "instance_healthy") else True
                nacos_instance_ephemeral = options.nacos_config.get("instance_ephemeral") if options.nacos_config.get(
                    "instance_ephemeral") else False
                nacos_instance_group_name = options.nacos_config.get("instance_group_name") if options.nacos_config.get(
                    "instance_group_name") else "DEFAULT_GROUP"
                nacos_client = nacos.NacosClient(server_addresses=options.nacos_config.get("server_addresses"),
                                                 namespace=nacos_namespace,
                                                 endpoint=options.nacos_config.get("endpoint"),
                                                 ak=options.nacos_config.get("ak"), sk=options.nacos_config.get("sk"),
                                                 username=options.nacos_config.get("username"),
                                                 password=options.nacos_config.get("password"))
                nacos_client.add_naming_instance(service_name=nacos_service_name, ip=nacos_instance_ip,
                                                 port=nacos_instance_port,
                                                 cluster_name=options.nacos_config.get("cluster_name"),
                                                 weight=nacos_instance_weight,
                                                 metadata=options.nacos_config.get("instance_metadata"),
                                                 enable=nacos_instance_enable, healthy=nacos_instance_healthy,
                                                 ephemeral=nacos_instance_ephemeral,
                                                 group_name=nacos_instance_group_name)
                logging.info(f'service={nacos_service_name} register success')

        except Exception as e:
            logging.info(f'service register fail,e:{e.__str__()}')
            traceback.print_exc()



from lesscode.utils.EsUtil import es_condition_by_terms, es_condition_by_wildcard


def format_param_tag(bool_should_more_list, especial_tag_list):
    bool_should_list = []
    if especial_tag_list is not None:
        for tag in especial_tag_list:

            # diy_tag
            if tag in ["国家级专精特新", "省级专精特新", "国家级单项冠军", "瞪羚"]:
                es_condition_by_terms(bool_should_list, "tags.diy_tag", [tag + "企业"])
            if tag in ["高新技术企业", "央企", "瞪羚企业", "中国企业500强"]:
                es_condition_by_terms(bool_should_list, "tags.diy_tag", [tag])
            if tag in ["单项冠军"]:
                es_condition_by_terms(bool_should_list, "tags.diy_tag", ["国家级单项冠军企业"])
            if tag in ["专精特新"]:
                es_condition_by_terms(bool_should_list, "tags.diy_tag", ["省级专精特新企业", "国家级专精特新企业"])
            # 上市信息
            if tag in ["A股上市"]:
                bool_should_list.append(
                    {"bool":
                        {"must": [
                            {"terms": {"tags.market_tag.block": ["主板上市", "科创板上市", "创业板上市"]}},
                            {"terms": {"tags.market_tag.status": ["已上市"]}}
                        ]
                        }
                    })
            if tag in ["新三板"]:
                es_condition_by_terms(bool_should_list, "tags.market_tag.status", ["新三板挂牌"])
            if tag in ["已上市", "排队上市", "已退市"]:
                es_condition_by_terms(bool_should_list, "tags.market_tag.status", [tag])
            if tag in ["主板上市", "创业板上市", "科创板上市", "新三板-基础层", "新三板-创新层", "新三板-精选层", "北交所"]:
                es_condition_by_terms(bool_should_list, "tags.market_tag.block", [tag])
            # 其他  -此类不标准，尽量不要使用
            if tag in ["小巨人", "一条龙"]:
                es_condition_by_wildcard(bool_should_list, "tags.national_tag.tag_name", tag)
            if tag in ["隐形冠军", "成长", "小巨人", "首台套", "雏鹰", "省级单项冠军"]:
                es_condition_by_wildcard(bool_should_list, "tags.province_tag.tag_name", tag)
            if tag in ["雏鹰"]:
                es_condition_by_wildcard(bool_should_list, "tags.city_tag.tag_name", tag)
            if tag in ["雏鹰"]:
                es_condition_by_wildcard(bool_should_list, "tags.district_tag.tag_name", tag)
            if tag in ["独角兽"]:
                es_condition_by_wildcard(bool_should_list, "tags.rank_tag.rank_name", tag)
            if tag in ["科技型中小企业"]:
                es_condition_by_terms(bool_should_list, "tags.certification.certification_name", [tag])
            if tag in ["规上企业"]:
                es_condition_by_terms(bool_should_list, "tags.nonpublic_tag", [tag])
    bool_should_more_list.append(bool_should_list)



# -*- coding: utf-8 -*-
# @Time    : 2022/8/10 11:32
# @Author  : navysummer
# @Email   : navysummer@yeah.net
import tornado.options
from requests import post, get


def upms_register(item, full_url):
    if tornado.options.options.rms_register_enable:
        client_name = item[3] if item[3] else tornado.options.options.application_name
        title = item[4] if item[4] else full_url
        en_name = item[5] if item[5] else full_url
        access = item[6] if item[6] else 0
        client_res = get(url=f"{tornado.options.options.rms_register_server}/upms/client/fetchall",
                         params={"client_name": client_name}).json()
        if not client_res.get("data"):
            post(url=f"{tornado.options.options.rms_register_server}/upms/client/insert",
                 json={"client_name": client_name})
            client_res = get(url=f"{tornado.options.options.rms_register_server}/upms/client/fetchall",
                             params={"client_name": client_name}).json()
        client_id = client_res.get("data")[0].get("id")
        resource_res = get(
            url=f"{tornado.options.options.rms_register_server}/upms/resource/fetchall",
            params={"client_name": client_name, "resource_type": 2, "url": full_url}).json()
        if not resource_res.get("data"):
            res = post(url=f"{tornado.options.options.rms_register_server}/upms/resource/insert",
                       json={"client_id": client_id, "resource_type": 2, "url": full_url, "label": title,
                             "en_name": en_name, "access": access}).json()
            if res.get("status") == "00000":
                print(f"add url={full_url} to rms success")
            else:
                print(f"add url={full_url} to rms fail")



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2022/3/2 9:14 上午
# Copyright (C) 2021 The lesscode Team
import json
import random

import requests
from requests.auth import HTTPBasicAuth


class EsRequest:

    def __init__(self, host, port, user, password):
        # 主机地址
        self.host = host
        # 端口号
        self.port = port
        # 用户名
        self.user = user
        # 密码
        self.password = password
        self.auth = HTTPBasicAuth(user, password)
        host_str = host.split(",")
        self.hosts = [host for host in host_str]

    def es_selector_way(self, url_func_str, param_dict, find_condition=None, request_way="post"):
        if request_way == "get":
            request_way = self.format_es_get
        else:
            request_way = self.format_es_post
        res = None
        # 随机打乱列表
        random.shuffle(self.hosts)
        for host in self.hosts:
            param_dict["host"] = host
            param_dict["port"] = self.port
            url = url_func_str(**param_dict)
            try:
                res = request_way(url, find_condition)
                if res.get("took"):
                    break
            except:
                continue
        return res

    def format_es_post(self, url, body, params=None):
        """
        发送http请求
        :param params:
        :param url:
        :param body:
        :return:
        """
        r = requests.post(
            url,
            data=json.dumps(body),
            headers={'content-type': "application/json"},
            params=params,
            auth=self.auth
        )
        res = r.json()
        return res

    def format_es_put(self, url, body, params=None):
        """
        发送http请求
        :param params:
        :param url:
        :param body:
        :return:
        """
        r = requests.put(
            url,
            params=params,
            data=json.dumps(body),
            headers={'content-type': "application/json"},
            auth=self.auth
        )
        res = r.json()
        return res

    def format_es_get(self, url, params=None):
        """
        发送http请求
        :param params:
        :param url:
        :param body:
        :return:
        """
        r = requests.get(
            url,
            params=params,
            headers={'content-type': "application/json"},
            auth=self.auth
        )
        res = r.json()
        return res

    def format_es_delete(self, url, body=None, params=None):
        """
        发送http请求
        :param params:
        :param url:
        :param body:
        :return:
        """
        r = requests.delete(
            url,
            data=json.dumps(body) if body else body,
            params=params,
            headers={'content-type': "application/json"},
            auth=self.auth
        )
        res = r.json()
        return res

    def format_scroll_url(self, host=None, port=None, route_key=None, scroll=None):
        return self.replace_url_kwargs("http://{host}{port}/{route_key}/_search?scroll={scroll}",
                                       {"host": host, "port": port, "route_key": route_key, "scroll": scroll})

    def format_scroll_id_url(self, host=None, port=None, ):
        return self.replace_url_kwargs("http://{host}{port}/_search/scroll",
                                       {"host": host, "port": port})

    def format_es_post_url(self, host=None, port=None, route_key=None):
        return self.replace_url_kwargs("http://{host}{port}/{route_key}/_search",
                                       {"host": host, "port": port, "route_key": route_key})

    def format_url(self, path, host=None, port=None):
        return self.replace_url_kwargs("http://{host}{port}{path}",
                                       {"host": host, "port": port, "path": path})

    def format_es_mapping_url(self, host=None, port=None, route_key=None):
        return self.replace_url_kwargs("http://{host}{port}/{route_key}/_mapping",
                                       {"host": host, "port": port, "route_key": route_key})

    def replace_url_kwargs(self, replace_url, replace_kwargs):
        if not replace_kwargs["host"]:
            replace_kwargs["host"] = self.host
        if not replace_kwargs["port"]:
            replace_kwargs["port"] = self.port
        if replace_kwargs.get("port"):
            replace_kwargs["port"] = f':{replace_kwargs["port"]}'
        else:
            replace_kwargs["port"] = ""
        replace_url = replace_url.format(**replace_kwargs)
        return replace_url

    def close(self):
        pass



import copy
import datetime
import functools
import inspect
import json
import logging
import sys
import traceback

from lesscode.db.redis.redis_helper import RedisHelper
from tornado.options import options

# 装饰器
from lesscode.utils.JsonUtil import JSONEncoder


def Cache(ex=3600 * 8, cache_key=None):
    def cache_func(func):
        # 默认key生成方法：str(item)
        @functools.wraps(func)
        def cache_wrapper(*args, **params):
            return deal_cache(func, ex, cache_key, *args, **params)

        return cache_wrapper

    return cache_func


def deal_cache(func, ex, cache_key, *args, **params):
    # 获取缓存查询key
    signature = inspect.signature(func)
    params = dict(sorted(params.items(), key=lambda x: x[0]))
    func_name = str(func).split(" ")[1]
    if not cache_key:
        cache_key = format_insert_key(signature, func_name, args, params)
    value = query_cache(cache_key)
    if value is not False:
        data = value
    else:
        start = datetime.datetime.now()
        logging.info("[组件：{}]数据开始计算！".format(func_name))
        copy_params = copy.deepcopy(params)
        data = func(*args, **copy_params)
        # 插入缓存表
        insert_cache(data, ex, cache_key)
        logging.info("[组件：{}]数据缓存已刷新！用时{}".format(func_name, datetime.datetime.now() - start))

    return data


def query_cache(cache_key, conn_name=None):
    if conn_name is None:
        conn_name = options.cache_conn
    if options.cache_enable:
        data = RedisHelper(conn_name).sync_get(cache_key)
        if data:
            value = json.loads(data)
            return value
        else:
            logging.info("str_select_key为".format(cache_key))
            return False
    return False


def format_insert_key(signature, func_name, args, params):
    _args = []
    param_keys = list(signature.parameters.keys())
    if param_keys and args:
        if param_keys[0] == "self":
            _args = copy.deepcopy(args[1:])
        else:
            _args = copy.deepcopy(args)
    if isinstance(_args, tuple):
        _args = list(_args)
    for k in params:
        _args.append(json.dumps(params[k]))
    str_insert_key = "&".join([str(x) for x in _args])
    str_insert_key = options.route_prefix + "#" + func_name + "#" + str_insert_key

    return str_insert_key


def insert_cache(data, ex, cache_key, conn_name=None):
    # 大于512kb不缓存
    if sys.getsizeof(data) <= 512 * 1024:
        if conn_name is None:
            conn_name = options.cache_conn
        if options.cache_enable:
            try:
                RedisHelper(conn_name).sync_set(cache_key, JSONEncoder().encode(data), ex=ex)
            except:
                logging.error(traceback.format_exc())



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2021/11/18 4:41 下午
import shutil

import setuptools
from setuptools.command.install_scripts import install_scripts
from lesscode.version import __version__

with open("README.md", "r") as fh:
    long_description = fh.read()


class InstallScripts(install_scripts):

    def run(self):
        setuptools.command.install_scripts.install_scripts.run(self)

        # Rename some script files
        for script in self.get_outputs():
            if script.endswith(".py"):
                dest = script[:-3]
            else:
                continue
            shutil.copy(script, dest)


setuptools.setup(
    name="lesscode-py",
    version=__version__,
    author="Chao.yy",
    author_email="yuyc@ishangqi.com",
    description="lesscode-python 是基于tornado的web开发脚手架项目，该项目初衷为简化开发过程，让研发人员更加关注业务。",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitee.com/yongchao9/lesscode-python",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        # "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    platforms='python',
    install_requires=[
        "tornado==6.0",
        "tornado-sqlalchemy==0.7.0",
        "aiomysql==0.0.22",
        "motor==2.5.1",
        "elasticsearch==7.15.2",
        "aiohttp==3.8.3",
        "crypto==1.4.1",
        "pycryptodome==3.12.0",
        "aioredis==2.0.1",
        "DBUtils==3.0.2",
        "redis==4.1.4",
        "requests==2.27.1",
        "neo4j==5.0.0",
        "snowland-smx==0.3.1",
        "py_eureka_client==0.11.3",
        "ks3sdk==1.5.0",
        "filechunkio==1.8",
        "APScheduler==3.9.1",
        "nacos-sdk-python==0.1.8",
        "pika==1.3.0",
        "kafka-python==2.0.2",
        "nebula3-python==3.4.0"
    ],
    scripts=['bin/lesscode.py'],

    cmdclass={
        "install_scripts": InstallScripts
    }

)
"""
        "aiopg>=1.3.3",
"""
"""
1、打包流程
打包过程中也可以多增加一些额外的操作，减少上传中的错误

# 先升级打包工具
pip install --upgrade setuptools wheel twine

# 打包
python setup.py sdist bdist_wheel

# 检查
twine check dist/*

# 上传pypi
twine upload dist/*
twine upload dist/* -u yuyc -p yu230225
twine upload dist/* -r https://pypi.chanyeos.com/pip_packages/common/simple/
# 安装最新的版本测试
pip install -U lesscode-py -i https://pypi.org/simple
"""



from copy import deepcopy


def get_value_from_dict(data: dict, key: str = None):
    value = deepcopy(data)
    if key:
        key_list = key.split(".")
        for k in key_list:
            if not isinstance(value, dict):
                break
            else:
                value = value.get(k)
    return value


# 去空
def eliminate_empty(list):
    return [x for x in list if x is not None]



from copy import deepcopy

from lesscode.utils.common import get_value_from_dict


def money_format(value, is_int=False):
    value = "%.2f" % float(value)
    components = str(value).split('.')
    if len(components) > 1:
        left, right = components
        right = '.' + right
    else:
        left, right = components[0], ''
    result = ''
    while left:
        result = left[-3:] + ',' + result
        left = left[:-3]
    if is_int:
        return result.strip(',')
    return result.strip(',') + right


def get_value_by_key(data: dict, key: str = None):
    if key:
        data = data.get(key)
    return data


def get_data_by_deep_from_dict(data: dict, deep: int):
    if not data:
        return {}
    result = {}
    if deep != 0:
        for _v in data.values():
            sub_result = get_data_by_deep_from_dict(deepcopy(_v), deep - 1)
            result.update(sub_result)
    else:
        return deepcopy(data)
    return result


def dict2ring(data: dict, unit: str = "", second_unit: str = "", key: str = "count", title: str = "", flag=True):
    """
    单层字典转饼图或者环形图
    :param flag: 百分比是否*100
    :param data: 需要转换的数据，实例1：{"测试1":{"count":1},"测试2":{"count":1}},实例2：{"测试1":1,"测试2":2}
    :param unit: 百分比单位
    :param second_unit: 数值单位
    :param key: 数据key
    :param title:图题
    :return:
    """
    total = 0
    for name, value in data.items():
        total += get_value_by_key(value, key)
    result = {
        "title": title,
        "data": [{"name": name,
                  "unit": unit,
                  "value": (money_format(get_value_by_key(value, key) / total * 100)) if total > 0 else 0,
                  "second_unit": second_unit,
                  "second_value": get_value_by_key(value, key)} for name, value in data.items()]
    }
    for name, value in data.items():
        value = (money_format(get_value_by_key(value, key) / total)) if total > 0 else 0
        if flag:
            value = value * 100
        result["data"].append({"name": name,
                               "unit": unit,
                               "value": value,
                               "second_unit": second_unit,
                               "second_value": get_value_by_key(value, key)})
    return result


def dict2columnar(data: dict, num: int, key_map: dict = None, title: str = "",
                  x_unit="", y_unit: str = "", data_key="count", default=None):
    """
    字典转柱状图，单柱或者多柱
    :param data: 字典数据
        实例1：{"河南省": 1, "北京市": 1}
        实例2：{"河南省": {"人工智能": 1, "传感器": 1}, "北京市": {"人工智能": 1, "传感器": 1}}
        实例3：{"河南省": {"人工智能": {"count":1}, "传感器": {"count":1}}, "北京市": {"人工智能": {"count":1}, "传感器": {"count":1}}}
    :param num: 柱子个数
    :param key_map: 字段映射
        实例1：{"人工智能": {"name": "人工智能", "data_key": "ren"}, "传感器": {"name": "传感器", "data_key": "chuan"}}
        实例2：{"人工智能": {"data_key": "ren"}, "传感器": {"data_key": "chuan"}}
    :param title: 图题
    :param x_unit: x轴单位
    :param y_unit: y轴单位
    :param data_key: 数据key
    :param default:获取不到内容时的默认数据
    :return:
    """
    first_keys = list(data.keys())
    result = {
        "title": title,
        "xAxis": first_keys,
        "yAxis": [],
        "xUnit": x_unit,
        "yUnit": y_unit
    }
    if num == 1:
        for first_key, first_value in data.items():
            result["xAxis"].append(first_key)
            value = get_value_by_key(first_value, data_key) if isinstance(first_value,
                                                                          dict) and data_key else first_value
            info = {"name": key_map.get(first_key, {}).get("name", first_key) if key_map else first_key,
                    "data_key": key_map.get(first_key, {}).get("data_key", first_key) if key_map else first_key,
                    "value": get_value_by_key(value, data_key) if isinstance(value, dict) and data_key else value}
            result["yAxis"].append(info)
    if num == 2:
        total_dict = {}
        names = []
        for first_key, first_value in data.items():
            unit_dict = {}

            for second_key, second_value in first_value.items():
                unit_dict[second_key] = get_value_by_key(second_value, data_key) \
                    if isinstance(second_value, dict) and data_key else second_value
                if second_key not in names:
                    names.append(second_key)
            total_dict[first_key] = unit_dict
        for name in names:
            info = {"name": key_map.get(name, {}).get("name", name) if key_map else name,
                    "data_key": key_map.get(name, {}).get("data_key", name) if key_map else name,
                    "value": []}
            for first_key in first_keys:
                info["value"].append(total_dict.get(first_key, {}).get(name, default))
            result["yAxis"].append(info)
    return result


def list2table(title: str, head, data: list, column_keys: dict = None, index_dict: dict = None):
    """
    list数据转换成表格
    :param index_dict: 索引列信息，实例：{"start":1,"column_name":"序号"，"index"："index"，"key"："index"}
    :param column_keys:数据对应点的keys，实例1：{"name1":"name2"} 说明：name1：接口返回字段名，name2:是数据的key,支持多层key，通过.连接，例如：basic.name
    :param data: es返回的数据
    :param title: 表格标题，字符串类型
    :param head: 表头，支持dict和list ，实例1：{"企业名称": "name"},实例2：[{"企业名称": "name"}]

    """

    result = {
        "title": title,
        "columns": [],
        "dataSource": [],
        "total": 0
    }
    if isinstance(head, dict):
        for key, value in head.items():
            result["columns"].append({"title": key,
                                      "dataIndex": value, "key": value})
    elif isinstance(head, list):
        result["columns"] = head
    else:
        raise Exception(f'head={head} is error')
    if column_keys:
        for item in data:
            data_item = {
            }
            for key, value in column_keys.items():
                data_item[key] = get_value_from_dict(item, value)
            result["dataSource"].append(data_item)
    else:
        result["dataSource"] = data
    result["total"] = len(result["dataSource"])

    if index_dict:
        ix = index_dict.get("start", 1)
        index_title = index_dict.get("column_name", "序号")
        data_index = index_dict.get("index", "index")
        data_key = index_dict.get("key", "index")
        index_no = index_dict.get("index_no", 0)
        result["columns"].insert(index_no, {"title": index_title,
                                            "dataIndex": data_index, "key": data_key})
        for item in result["dataSource"]:
            item[data_key] = ix
            ix += 1

    return result


def convert(value, func):
    return func(value)


def list2table_with_page(title: str, head, data: list, column_keys: dict = None, index_dict: dict = None,
                         page_num: int = 1, page_size: int = 10, total: int = 0, column_covert: dict = None):
    """
    list数据转换成表格
    :param column_covert: {"id":str}
    :param total: 数据总数
    :param page_size: 每页数量
    :param page_num: 页码
    :param index_dict: 索引列信息，实例：{"start":1,"column_name":"序号"，"index"："index"，"key"："index"}
    :param column_keys:数据对应点的keys，实例1：{"name1":"name2"} 说明：name1：接口返回字段名，name2:是数据的key,支持多层key，通过.连接，例如：basic.name
    :param data: es返回的数据
    :param title: 表格标题，字符串类型
    :param head: 表头，支持dict和list ，实例1：{"企业名称": "name"},实例2：[{"企业名称": "name"}]

    """

    result = {
        "title": title,
        "columns": [],
        "dataSource": [],
        "total": 0
    }
    if isinstance(head, dict):
        for key, value in head.items():
            result["columns"].append({"title": key,
                                      "dataIndex": value, "key": value})
    elif isinstance(head, list):
        result["columns"] = head
    else:
        raise Exception(f'head={head} is error')
    if column_keys:
        for item in data:
            data_item = {
            }
            for key, value in column_keys.items():
                new_value = get_value_from_dict(item, value)
                if column_covert:
                    if key in column_covert:
                        new_value = convert(new_value, column_covert.get(key))
                data_item[key] = new_value
            result["dataSource"].append(data_item)
    else:
        result["dataSource"] = data
    result["total"] = total

    if index_dict:
        ix = index_dict.get("start", 1) + (page_num - 1) * page_size
        index_title = index_dict.get("column_name", "序号")
        data_index = index_dict.get("index", "index")
        data_key = index_dict.get("key", "index")
        index_no = index_dict.get("index_no", 0)
        result["columns"].insert(index_no, {"title": index_title,
                                            "dataIndex": data_index, "key": data_key})
        for item in result["dataSource"]:
            item[data_key] = ix
            ix += 1

    return result



# -*- coding: utf-8 -*-
# author:chao.yy
# email:yuyc@ishangqi.com
# date:2022/2/17 2:45 下午
# Copyright (C) 2022 The lesscode Team
import asyncio

import pymysql
from dbutils.pooled_db import PooledDB
from tornado.options import options, define
from sqlalchemy import create_engine

from lesscode.db.base_connection_pool import BaseConnectionPool
import aiomysql


class SqlAlchemyPool(BaseConnectionPool):
    """
    mysql 数据库链接创建类
    """

    def sync_create_pool(self):
        db_type = "mysql"
        if self.conn_info.params:
            if self.conn_info.params.get("db_type"):
                db_type = self.conn_info.params.pop("db_type")
        if db_type == "mysql":
            url = 'mysql+pymysql://{}:{}@{}:{}/{}?charset=utf8mb4'.format(
                self.conn_info.user, self.conn_info.password, self.conn_info.host, self.conn_info.port,
                self.conn_info.db_name)
        elif db_type == "postgresql":
            url = 'postgresql+psycopg2://{}:{}@{}:{}/{}?charset=utf8mb4'.format(
                self.conn_info.user, self.conn_info.password, self.conn_info.host, self.conn_info.port,
                self.conn_info.db_name)
        else:
            raise Exception("UNSUPPORTED DB TYPE")
        engine = create_engine(url, echo=self.conn_info.params.get("echo",
                                                                   True) if self.conn_info.params else True,
                               pool_size=self.conn_info.min_size,
                               pool_recycle=self.conn_info.params.get("pool_recycle",
                                                                      3600) if self.conn_info.params else 3600,
                               max_overflow=self.conn_info.params.get("max_overflow",
                                                                      0) if self.conn_info.params else 0,
                               pool_timeout=self.conn_info.params.get("pool_timeout",
                                                                      10) if self.conn_info.params else 10,
                               pool_pre_ping=self.conn_info.params.get("pool_pre_ping",
                                                                       True) if self.conn_info.params else True)
        return engine
